Package: DRIMSeq
Type: Package
Title: Differential transcript usage and tuQTL analyses with
        Dirichlet-multinomial model in RNA-seq
Version: 1.40.0
Date: 2017-05-24
Authors@R: c(person(given = "Malgorzata", family = "Nowicka", 
    role = c("aut", "cre"),
    email = "gosia.nowicka.uzh@gmail.com"))
Description: The package provides two frameworks. One for the differential
    transcript usage analysis between different conditions and one for the tuQTL analysis.
    Both are based on modeling the counts of genomic features (i.e., transcripts) with the Dirichlet-multinomial distribution. The package also
    makes available functions for visualization and exploration of the data and
    results.
biocViews: ImmunoOncology, SNP, AlternativeSplicing,
        DifferentialSplicing, Genetics, RNASeq, Sequencing,
        WorkflowStep, MultipleComparison, GeneExpression,
        DifferentialExpression
License: GPL (>= 3)
Depends: R (>= 3.4.0)
Imports: utils, stats, MASS, GenomicRanges, IRanges, S4Vectors,
        BiocGenerics, methods, BiocParallel, limma, edgeR, ggplot2,
        reshape2
Suggests: PasillaTranscriptExpr, GeuvadisTranscriptExpr, grid,
        BiocStyle, knitr, testthat
ByteCompile: false
VignetteBuilder: knitr
Collate: 'DRIMSeq.R' 'class_show_utils.R' 'class_MatrixList.R'
        'class_dmDSdata.R' 'class_dmDSprecision.R' 'class_dmDSfit.R'
        'class_dmDStest.R' 'class_dmSQTLdata.R'
        'class_dmSQTLprecision.R' 'class_dmSQTLfit.R'
        'class_dmSQTLtest.R' 'dmDS_CRadjustment.R'
        'dmDS_estimateCommonPrecision.R'
        'dmDS_estimateTagwisePrecision.R' 'dmDS_filter.R' 'dmDS_fit.R'
        'dmDS_profileLik.R' 'dmSQTL_CRadjustment.R'
        'dmSQTL_estimateCommonPrecision.R'
        'dmSQTL_estimateTagwisePrecision.R' 'dmSQTL_filter.R'
        'dmSQTL_fit.R' 'dmSQTL_permutations.R' 'dmSQTL_profileLik.R'
        'dm_CRadjustmentManyGroups.R' 'dm_CRadjustmentOneGroup.R'
        'dm_CRadjustmentRegression.R' 'dm_LRT.R' 'dm_core_Hessian.R'
        'dm_core_colorb.R' 'dm_core_deviance.R' 'dm_core_lik.R'
        'dm_core_score.R' 'dm_estimateMeanExpression.R'
        'dm_fitManyGroups.R' 'dm_fitOneGroup.R' 'dm_fitRegression.R'
        'dm_plotData.R' 'dm_plotPrecision.R' 'dm_plotProportions.R'
        'dm_plotPvalues.R' 'dm_profileLikModeration.R'
RoxygenNote: 6.0.1
git_url: https://git.bioconductor.org/packages/DRIMSeq
git_branch: RELEASE_3_23
git_last_commit: da1d6e4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:52:25 UTC; biocbuild
Author: Malgorzata Nowicka [aut, cre]
Maintainer: Malgorzata Nowicka <gosia.nowicka.uzh@gmail.com>
