# Generated by roxygen2: do not edit by hand

export("common_precision<-")
export("genewise_precision<-")
export(common_precision)
export(dmDSdata)
export(dmFilter)
export(dmFit)
export(dmPrecision)
export(dmSQTLdata)
export(dmTest)
export(genewise_precision)
export(mean_expression)
export(plotData)
export(plotPValues)
export(plotPrecision)
export(plotProportions)
export(proportions)
export(results)
export(samples)
exportMethods("$")
exportMethods("[")
exportMethods("[[")
exportMethods("colnames<-")
exportMethods("common_precision<-")
exportMethods("genewise_precision<-")
exportMethods("names<-")
exportMethods("rownames<-")
exportMethods(coefficients)
exportMethods(colnames)
exportMethods(common_precision)
exportMethods(counts)
exportMethods(design)
exportMethods(dim)
exportMethods(dmFilter)
exportMethods(dmFit)
exportMethods(dmPrecision)
exportMethods(dmTest)
exportMethods(elementNROWS)
exportMethods(genewise_precision)
exportMethods(length)
exportMethods(mean_expression)
exportMethods(names)
exportMethods(ncol)
exportMethods(nrow)
exportMethods(plotData)
exportMethods(plotPValues)
exportMethods(plotPrecision)
exportMethods(plotProportions)
exportMethods(proportions)
exportMethods(results)
exportMethods(rownames)
exportMethods(samples)
import(BiocGenerics)
import(BiocParallel)
import(GenomicRanges)
import(edgeR)
import(methods)
importFrom(IRanges,width)
importFrom(S4Vectors,elementNROWS)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_jitterdodge)
importFrom(ggplot2,scale_colour_gradient)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(limma,nonEstimable)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,complete.cases)
importFrom(stats,constrOptim)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,nlm)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,optimHess)
importFrom(stats,optimize)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(utils,head)
importFrom(utils,tail)
